package main

import (
	"fmt"
	"os"

	"github.com/spf13/viper"
	gl "github.com/xanzy/go-gitlab"
)

type Config struct {
	token   string
	baseurl string
}

func ReadConfig(conf string) *Config {
	f, err := os.Open(conf)
	if err != nil {
		panic(fmt.Errorf("failed to parse config file %s", err))
	}
	defer f.Close()
	viper.SetConfigType("hcl")
	err = viper.ReadConfig(f)
	c := &Config{
		token:   viper.GetString("token"),
		baseurl: viper.GetString("baseurl"),
	}

	if err != nil {
		panic(fmt.Errorf("Fatal error config file: %s \n", err))
	}

	return c
}

func newProjectList(client *gl.Client) func() (*gl.Project, error) {
	i := 0
	opt := &gl.ListProjectsOptions{
		ListOptions: gl.ListOptions{
			Page:    1,
			PerPage: 20,
		},
		Archived:   nil,
		OrderBy:    nil,
		Sort:       nil,
		Search:     nil,
		Simple:     nil,
		Visibility: nil,
	}
	projects, resp, err := client.Projects.ListProjects(opt)

	return func() (*gl.Project, error) {
		if projects != nil && i >= len(projects) {
			if resp.NextPage == 0 {
				return nil, nil
			}
			opt.ListOptions.Page++
			i = 0
			projects, resp, err = client.Projects.ListProjects(opt)
		}
		if err == nil {
			i++
			return projects[i-1], nil
		} else {
			return nil, err
		}
	}
}

func main() {
	c := ReadConfig("config.conf")

	client := gl.NewClient(nil, c.token)
	client.SetBaseURL(c.baseurl)

	i := 0
	gen := newProjectList(client)
	for p, _ := gen(); p != nil; p, _ = gen() {
		fmt.Printf("% 3d: %s / %s\n", i, p.Path, p.Name)
		i++
	}
}
